package com.mortals.xhx.module.help.web;

import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.annotation.RepeatSubmit;
import com.mortals.framework.common.IBaseEnum;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.common.code.EvaluateType;
import com.mortals.xhx.common.key.Constant;
import com.mortals.xhx.module.customer.model.CustomerEntity;
import com.mortals.xhx.module.customer.service.CustomerService;
import com.mortals.xhx.module.help.model.HelpEvaluateEntity;
import com.mortals.xhx.module.help.service.HelpEvaluateService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
*
* 帮助评价信息
*
* @author zxfei
* @date 2022-06-29
*/
@RestController
@RequestMapping("help/evaluate")
public class HelpEvaluateController extends BaseCRUDJsonBodyMappingController<HelpEvaluateService,HelpEvaluateEntity,Long> {

    @Autowired
    private CustomerService customerService;

    public HelpEvaluateController(){
        super.setModuleDesc( "帮助评价信息");
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        this.addDict(model, "evaluateType", IBaseEnum.getEnumMap(EvaluateType.class));
        super.init(model, context);
    }

    @Override
    protected void saveBefore(HelpEvaluateEntity entity, Map<String, Object> model, Context context) throws AppException {
        if(this.getCurUser()==null||this.getCurUser().getUserType()!= Constant.CUSTOMER_USER){
            throw new AppException("非法用户,不可访问");
        }
        CustomerEntity customerEntity = customerService.get(this.getCurUser().getId());
        entity.setCustomerId(this.getCurUser().getId());
        entity.setCustName(customerEntity.getCustName());
        entity.setContactTelphone(customerEntity.getContactTelphone());
        entity.setOrganization(customerEntity.getOrganization());
    }

    @Override
    protected void doListBefore(HelpEvaluateEntity query, Map<String, Object> model, Context context) throws AppException {
        Map<String,String> orderCols = new HashMap<>();
        orderCols.put("evaluateTime","DESC");
        query.setOrderCols(orderCols);
    }

    @Override
    protected int doListAfter(HelpEvaluateEntity query, Map<String, Object> model, Context context) throws AppException {
        List<HelpEvaluateEntity> evaluateEntities = (List<HelpEvaluateEntity>)model.get("data");
        Map<Integer, List<HelpEvaluateEntity>> map = evaluateEntities.stream().collect(Collectors.groupingBy(HelpEvaluateEntity::getEvaluateType));
        Map<String, Object> dataMap = new HashMap<>();
        dataMap.put("useful",map.get(EvaluateType.USEFUL.getValue()));
        dataMap.put("useless",map.get(EvaluateType.USELESS.getValue()));
        model.put("data",dataMap);
        return 1;
    }

    /**
     * 新增有用评价
     * @return
     */
    @GetMapping({"useful"})
    public String useful(Long helpId) {
        Map<String, Object> model = new HashMap();
        Context context = this.getContext();
        int code = 1;
        String busiDesc = "保存有用" + this.getModuleDesc();
        HelpEvaluateEntity entity = new HelpEvaluateEntity();
        entity.setHelpId(helpId);
        entity.setEvaluateType(EvaluateType.USEFUL.getValue());
        entity.setEvaluateTime(new Date());
        try {
            this.saveBefore(entity, model, context);
            this.service.save(entity, context);
            model.put("id", entity.getId());
            code = this.saveAfter(entity, model, context);
            model.put("entity", entity);
            model.put("message_info", busiDesc + "成功");
            this.recordSysLog(this.request, busiDesc + " 【成功】 [id:" + entity.getId() + "]");
        } catch (Exception var7) {
            this.doException(this.request, busiDesc, model, var7);
            model.put("entity", entity);
            this.init(model, context);
            code = this.saveException(entity, model, context, var7);
        }

        this.init(model, context);
        JSONObject ret = new JSONObject();
        ret.put("code", code);
        ret.put("msg", model.remove("message_info"));
        ret.put("data", model);
        return ret.toJSONString();
    }

    /**
     * 新增有用评价
     * @return
     */
    @GetMapping({"useless"})
    public String useless(Long helpId) {
        Map<String, Object> model = new HashMap();
        Context context = this.getContext();
        int code = 1;
        String busiDesc = "保存无用" + this.getModuleDesc();
        HelpEvaluateEntity entity = new HelpEvaluateEntity();
        entity.setHelpId(helpId);
        entity.setEvaluateType(EvaluateType.USELESS.getValue());
        entity.setEvaluateTime(new Date());
        try {
            this.saveBefore(entity, model, context);
            this.service.save(entity, context);
            model.put("id", entity.getId());
            code = this.saveAfter(entity, model, context);
            model.put("entity", entity);
            model.put("message_info", busiDesc + "成功");
            this.recordSysLog(this.request, busiDesc + " 【成功】 [id:" + entity.getId() + "]");
        } catch (Exception var7) {
            this.doException(this.request, busiDesc, model, var7);
            model.put("entity", entity);
            this.init(model, context);
            code = this.saveException(entity, model, context, var7);
        }

        this.init(model, context);
        JSONObject ret = new JSONObject();
        ret.put("code", code);
        ret.put("msg", model.remove("message_info"));
        ret.put("data", model);
        return ret.toJSONString();
    }
}