package com.mortals.xhx.module.home.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.util.DateUtils;
import com.mortals.xhx.common.code.EvaluateType;
import com.mortals.xhx.common.code.PictureTypeEnum;
import com.mortals.xhx.common.code.ReplyStatusEnum;
import com.mortals.xhx.module.customer.dao.CustomerDao;
import com.mortals.xhx.module.customer.model.CustomerEntity;
import com.mortals.xhx.module.customer.model.CustomerQuery;
import com.mortals.xhx.module.design.dao.DesignMasterplateDao;
import com.mortals.xhx.module.design.model.DesignMasterplateEntity;
import com.mortals.xhx.module.font.dao.FontMaterialDao;
import com.mortals.xhx.module.font.model.FontMaterialEntity;
import com.mortals.xhx.module.help.dao.HelpDao;
import com.mortals.xhx.module.help.dao.HelpEvaluateDao;
import com.mortals.xhx.module.help.model.HelpEntity;
import com.mortals.xhx.module.help.model.HelpEvaluateEntity;
import com.mortals.xhx.module.home.model.MaterialStatisticsVO;
import com.mortals.xhx.module.picture.dao.PictureMaterialDao;
import com.mortals.xhx.module.proposal.dao.ProposalDao;
import com.mortals.xhx.module.proposal.model.ProposalEntity;
import com.mortals.xhx.module.question.dao.QuestionDao;
import com.mortals.xhx.module.question.model.QuestionEntity;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.DecimalFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.stream.Collectors;

@Service
public class HomePageService {

    @Autowired
    private PictureMaterialDao pictureMaterialDao;
    @Autowired
    private FontMaterialDao fontMaterialDao;
    @Autowired
    private DesignMasterplateDao designMasterplateDao;
    @Autowired
    private QuestionDao questionDao;
    @Autowired
    private HelpDao helpDao;
    @Autowired
    private HelpEvaluateDao helpEvaluateDao;
    @Autowired
    private ProposalDao proposalDao;
    @Autowired
    private CustomerDao customerDao;

    public Map<String, Object> customerStatistics() throws AppException {
        List<CustomerEntity> sumList = customerDao.getList(new CustomerEntity());
        CustomerQuery query = new CustomerQuery();
        query.setCreateTimeStart(DateUtils.addCurrDateToStr("yyyy-MM-dd",-1)+" 00:00:01");
        List<CustomerEntity> yesterdayList = customerDao.getList(query);
        Map<String, Object> map = new HashMap<>();
        Map<Integer,Long> sumMap = sumList.stream().collect(Collectors.groupingBy(CustomerEntity::getMemberLevel,Collectors.counting()));
        if(!sumMap.containsKey(1)){
            sumMap.put(1,0l);
        }
        if(!sumMap.containsKey(2)){
            sumMap.put(2,0l);
        }
        if(!sumMap.containsKey(3)){
            sumMap.put(3,0l);
        }
        map.put("customerTotal",sumList.size());
        map.put("trialTotal",sumMap.get(1));
        map.put("vipTotal",sumMap.get(2));
        map.put("designTotal",sumMap.get(3));
        if(CollectionUtils.isEmpty(yesterdayList)){
            map.put("totalAdd",0);
            map.put("totalChange","0.00%");
            map.put("trialAdd",0);
            map.put("trialChange","0.00%");
            map.put("vipAdd",0);
            map.put("vipChange","0.00%");
            map.put("designAdd",0);
            map.put("designChange","0.00%");
        }else {
            Map<Integer,Long> yMap = yesterdayList.stream().collect(Collectors.groupingBy(CustomerEntity::getMemberLevel,Collectors.counting()));
            if(!yMap.containsKey(1)){
                yMap.put(1,0l);
            }
            if(!yMap.containsKey(2)){
                yMap.put(2,0l);
            }
            if(!yMap.containsKey(3)){
                yMap.put(3,0l);
            }
            map.put("totalAdd",yesterdayList.size());
            map.put("totalChange",percentage((long)yesterdayList.size(),(long)sumList.size()));
            map.put("trialAdd",yMap.get(1));
            map.put("trialChange",percentage(yMap.get(1),sumMap.get(1)));
            map.put("vipAdd",yMap.get(2));
            map.put("vipChange",percentage(yMap.get(2),sumMap.get(2)));
            map.put("designAdd",yMap.get(3));
            map.put("designChange",percentage(yMap.get(3),sumMap.get(3)));
        }
        return map;
    }

    private static String percentage(Long num1,Long num2){
        String rate="0.00%";
        String format="0.00";
        if(num2 != 0 && num1 != 0){
            DecimalFormat dec = new DecimalFormat(format);
            rate =  dec.format((double) num1 / num2 * 100)+"%";
            while(true){
                if(rate.equals(format+"%")){
                    format=format+"0";
                    DecimalFormat dec1 = new DecimalFormat(format);
                    rate =  dec1.format((double) num1 / num2 *100)+"%";
                }else {
                    break;
                }
            }
        }else if(num1 != 0 && num2 == 0){
            rate = "100%";
        }
        return rate;
    }

    public Map<String,Object> resourceCensus(){
        Map<String,Object> result = new HashMap<>();
        List<MaterialStatisticsVO> materialStatisticsVOS = pictureMaterialDao.selectPictureCensus();
        if(CollectionUtils.isNotEmpty(materialStatisticsVOS)){
            materialStatisticsVOS.forEach(e->{
                if(e.getPictureType()== PictureTypeEnum.PICTURE.getValue()){
                    result.put("pictureCount",e.getPictureCount());
                }
                if(e.getPictureType()== PictureTypeEnum.ELEMENT.getValue()){
                    result.put("elementCount",e.getPictureCount());
                }
                if(e.getPictureType()== PictureTypeEnum.BACKGROUND.getValue()){
                    result.put("backgroundCount",e.getPictureCount());
                }
            });
        }
        if(!result.containsKey("pictureCount")){
            result.put("pictureCount",0);
        }
        if(!result.containsKey("elementCount")){
            result.put("elementCount",0);
        }
        if(!result.containsKey("backgroundCount")){
            result.put("backgroundCount",0);
        }
        int fontCount = fontMaterialDao.getCount(new FontMaterialEntity());
        result.put("fontCount",fontCount);
        int designCount = designMasterplateDao.getCount(new DesignMasterplateEntity());
        result.put("masterplate",designCount);
        DesignMasterplateEntity query = new DesignMasterplateEntity();
        Map<String,String> orderCols = new HashMap<>();
        orderCols.put("masterplateUseNum", "DESC");
        orderCols.put("createTime", "DESC");
        query.setOrderCols(orderCols);
        query.setPage(1);
        query.setSize(5);
        List<DesignMasterplateEntity> list = designMasterplateDao.getList(query,0,5);
        result.put("top5",list);
        return result;
    }

    public Map<String,Object> feedbackCensus(){
        Map<String,Object> result = new HashMap<>();
        int questionCount = questionDao.getCount(new QuestionEntity());
        result.put("questionCount",questionCount);
        int helpCount = helpDao.getCount(new HelpEntity());
        result.put("helpCount",helpCount);
        HelpEvaluateEntity hq = new HelpEvaluateEntity();
        hq.setEvaluateType(EvaluateType.USEFUL.getValue());
        int helpUsefulCount = helpEvaluateDao.getCount(hq);
        result.put("helpUseful",percentage((long)helpUsefulCount,(long)helpCount));
        hq.setEvaluateType(EvaluateType.USELESS.getValue());
        int helpUselessCount = helpEvaluateDao.getCount(hq);
        result.put("helpUseless",percentage((long)helpUselessCount,(long)helpCount));
        int proposalCount = proposalDao.getCount(new ProposalEntity());
        result.put("proposalCount",proposalCount);
        ProposalEntity pquery = new ProposalEntity();
        pquery.setReplyStatus(ReplyStatusEnum.REPLY.getValue());
        int proposalReplyCount = proposalDao.getCount(pquery);
        result.put("proposalReplyCount",proposalReplyCount);
        return result;
    }

    public JSONArray creatData(){
        JSONArray list = new JSONArray();
        Random random = new Random();
        random.setSeed(10000L);
        int i=0;
        while (true){
            if(i>=30){
                break;
            }
            String dateTime = DateUtils.addCurrDateToStr("yyyy-MM-dd",(0-i));
            JSONObject json = new JSONObject();
            json.put("dateTime",dateTime);
            json.put("count",random.nextInt(1000));
            list.add(json);
            i++;
        }
        return list;
    }

    public JSONArray creatCustomerData() {
        JSONArray list = new JSONArray();
        Random random = new Random();
        random.setSeed(10000L);
        JSONObject json = new JSONObject();
        json.put("areaName","成都");
        json.put("count",random.nextInt(1000));
        list.add(json);
        JSONObject json1 = new JSONObject();
        json1.put("areaName","眉山");
        json1.put("count",random.nextInt(1000));
        list.add(json1);
        JSONObject json2 = new JSONObject();
        json2.put("areaName","绵阳");
        json2.put("count",random.nextInt(1000));
        list.add(json2);
        JSONObject json3 = new JSONObject();
        json3.put("areaName","宜宾");
        json3.put("count",random.nextInt(1000));
        list.add(json3);
        JSONObject json4 = new JSONObject();
        json4.put("areaName","乐山");
        json4.put("count",random.nextInt(1000));
        list.add(json4);
        JSONObject json5 = new JSONObject();
        json5.put("areaName","雅安");
        json5.put("count",random.nextInt(1000));
        list.add(json5);
        return list;
    }
}
