package com.mortals.xhx.module.question.web;

import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.common.IBaseEnum;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.service.IUser;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.common.code.QuestionRecordType;
import com.mortals.xhx.common.key.Constant;
import com.mortals.xhx.module.customer.model.CustomerEntity;
import com.mortals.xhx.module.customer.service.CustomerService;
import com.mortals.xhx.module.question.model.QuestionEntity;
import com.mortals.xhx.module.question.model.QuestionRecordEntity;
import com.mortals.xhx.module.question.service.QuestionRecordService;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;
/**
*
* 问答记录信息
*
* @author zxfei
* @date 2022-06-28
*/
@RestController
@RequestMapping("question/record")
public class QuestionRecordController extends BaseCRUDJsonBodyMappingController<QuestionRecordService,QuestionRecordEntity,Long> {

    @Autowired
    private CustomerService customerService;

    public QuestionRecordController(){
        super.setModuleDesc( "问答记录信息");
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        this.addDict(model, "recordType", IBaseEnum.getEnumMap(QuestionRecordType.class));
        super.init(model, context);
    }

    @Override
    protected void doListBefore(QuestionRecordEntity query, Map<String, Object> model, Context context) throws AppException {
        Map<String,String> orderCols = new HashMap<>();
        orderCols.put("createTime","DESC");
        query.setOrderCols(orderCols);
    }

    @PostMapping({"ask"})
    public String ask(@RequestBody QuestionRecordEntity entity) {

        Map<String, Object> model = new HashMap();
        Context context = this.getContext();
        String busiDesc = "新增咨询" + this.getModuleDesc();
        int code = 1;
        try {
            if(this.getCurUser()==null||this.getCurUser().getUserType()!= Constant.CUSTOMER_USER){
                throw new AppException("非法用户,不可访问");
            }
            CustomerEntity customerEntity = customerService.get(this.getCurUser().getId());
            entity.setCustomerId(this.getCurUser().getId());
            entity.setCustName(customerEntity.getCustName());
            entity.setContactTelphone(customerEntity.getContactTelphone());
            entity.setOrganization(customerEntity.getOrganization());

            entity = this.service.ask(entity,context);
            model.put("entity", entity);
            model.put("message_info", busiDesc + "成功");
            this.recordSysLog(this.request, busiDesc + " 【成功】 [id:" + entity.getId() + "]");
        } catch (Exception var7) {
            this.doException(this.request, busiDesc, model, var7);
            model.put("entity", entity);
            this.init(model, context);
            code = this.saveException(entity, model, context, var7);
        }

        this.init(model, context);
        JSONObject ret = new JSONObject();
        ret.put("code", code);
        ret.put("msg", model.remove("message_info"));
        ret.put("data", model);
        return ret.toJSONString();
    }

    @PostMapping({"answer"})
    public String answer(@RequestBody QuestionRecordEntity entity) {

        Map<String, Object> model = new HashMap();
        Context context = this.getContext();
        String busiDesc = "新增回复" + this.getModuleDesc();
        int code = 1;
        try {
            if(this.getCurUser()==null){
                throw new AppException("用户未登录或登录失效，请重新登录");
            }
            entity.setCustName(this.getCurUser().getRealName());
            entity = this.service.answer(entity,context);
            model.put("entity", entity);
            model.put("message_info", busiDesc + "成功");
            this.recordSysLog(this.request, busiDesc + " 【成功】 [id:" + entity.getId() + "]");
        } catch (Exception var7) {
            this.doException(this.request, busiDesc, model, var7);
            model.put("entity", entity);
            this.init(model, context);
            code = this.saveException(entity, model, context, var7);
        }

        this.init(model, context);
        JSONObject ret = new JSONObject();
        ret.put("code", code);
        ret.put("msg", model.remove("message_info"));
        ret.put("data", model);
        return ret.toJSONString();
    }
}